/*
   Vector Class Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to create a class that will represent a vertex x, y,
   and z postion.  This class will be used a lot and have a lot of function that
   perform vector math and operations.  The purpose of this is to expose you to this
   class before it is used.
*/


// Demo: Vector Class.


#define WIN32_LEAN_AND_MEAN// Trims down the libraries.
#define VC_LEANMEAN        // Trims even further.


#include<windows.h>        // Windows header file.
#include<gl/gl.h>          // OpenGL header.
#include<gl/glu.h>         // OpenGL utility.
#include"CVector.h"        // Holds the Vector class.


// Function prototypes...
bool InitializeGL();       // Initialize OpenGL.
void RenderScene();        // Renders each frame.


// The first is a global HDC that will be used in the RenderScene() function to swap buffers.
HDC g_HDC;

// Since a CVector4 object holds a single point's 3D data so lets create an array of 3 for each
// triangle point.
CVector4 Triangle[3];


void SetupPixelFormat(HDC hDC)
{
   int nPixelFormat;

   static PIXELFORMATDESCRIPTOR pfd = {
         sizeof(PIXELFORMATDESCRIPTOR),   // size of structure.
         1,                               // always 1.
         PFD_DRAW_TO_WINDOW |             // support window
         PFD_SUPPORT_OPENGL |             // support OpenGl
         PFD_DOUBLEBUFFER,                // support double buffering
         PFD_TYPE_RGBA,                   // support RGBA
         16,                              // bit color mode
         0, 0, 0, 0, 0, 0,                // ignore color bits
         0,                               // no alpha buffer
         0,                               // ignore shift bit
         0,                               // no accumulation buffer
         0, 0, 0, 0,                      // ignore accumulation bits.
         16,                              // number of depth buffer bits.
         0,                               // number of stencil buffer bits.
         0,                               // 0 means no auxiliary buffer
         PFD_MAIN_PLANE,                  // The main drawing plane
         0,                               // this is reserved
         0, 0, 0 };                       // layer masks ignored.

   // this chooses the best pixel format and returns index.
   nPixelFormat = ChoosePixelFormat(hDC, &pfd);

   // This set pixel format to device context.
   SetPixelFormat(hDC, nPixelFormat, &pfd);
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static HGLRC hRC;                         // Rendering context.
   static HDC hDC;                           // Device context.
   int width, height;                        // The window width and height.


   switch(message)
      {
         case WM_CREATE:                     // Windows creation.
            hDC = GetDC(hwnd);               // This gets the device context for our window.
            g_HDC = hDC;                     // Assigns the global device context to this one.
            SetupPixelFormat(hDC);           // Call the pixel format function.

            hRC = wglCreateContext(hDC);     // Creates the rendering context.
            wglMakeCurrent(hDC, hRC);        // Makes the rendering context.
            return 0;
            break;

         case WM_CLOSE:                      // Close message.
         case WM_DESTROY:
            wglMakeCurrent(hDC, NULL);
            wglDeleteContext(hRC);           // Deletes the rendering context.

            PostQuitMessage(0);              // Says close the program.
            return 0;
            break;

         case WM_SIZE:                       // re-size message.
            height = HIWORD(lParam);         // This gets the height of the window.
            width = LOWORD(lParam);          // This gets the width of the window.

            if(height==0)                    // we don't want it to be possible for a
               {                             // height of 0.  If it is 0 me make it 1.
                  height = 1;
               }

            glViewport(0, 0, width, height); // resets the viewport to new dimensions.
            glMatrixMode(GL_PROJECTION);     // Sets the projection matrix.
            glLoadIdentity();                // Reset the modelview matrix.

            // calculate the aspect ratio of the window.
            gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f);

            glMatrixMode(GL_MODELVIEW);      // Sets the projection matrix.
            glLoadIdentity();                // Reset the modelview matrix.

            return 0;
            break;

         case WM_KEYDOWN:
		      switch(wParam)
               { 
                  // Close the app if the user presses escape.
		            case VK_ESCAPE:
                     PostQuitMessage(0);
				         break;
               }
            break;


         default: // Always have a default just in case.
            break;
      }

   // What this does is pass all of the unhandled messages to DefWindowProc.
   return (DefWindowProc(hwnd, message, wParam, lParam));
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   MSG msg;                // A message variable.
   WNDCLASSEX windowClass; // Your Window class.
   HWND hwnd;              // The Window handle.
   bool isFinished;        // This will be used to check if the program is done or not.

   // This is the Window class.
   windowClass.cbSize = sizeof(WNDCLASSEX);        // size of the WNDCLASSEX structure.
   windowClass.style = CS_HREDRAW | CS_VREDRAW;    // style of the window.
   windowClass.lpfnWndProc = WndProc;              // Address to the windows procedure.
   windowClass.cbClsExtra = 0;                     // Extra class information.
   windowClass.cbWndExtra = 0;                     // Extra window information.
   windowClass.hInstance = hInstance;              // Handle of application Instance.
   // Handle of application Icon.
   windowClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
   windowClass.hCursor = LoadCursor(NULL, IDC_ARROW);// mouse cursor
   windowClass.hbrBackground = NULL;               // background color.
   windowClass.lpszMenuName = NULL;                // name of the main menu.
   windowClass.lpszClassName = "UltimateGameProgrammingClass";// window class name.
   // icon when minimized.
   windowClass.hIconSm = LoadIcon(NULL, IDI_APPLICATION);

   // You must register you class with Windows.
   if(!RegisterClassEx(&windowClass)) return 0;

   // Create the window.
   hwnd = CreateWindowEx(NULL,// The extended window style.
                        "UltimateGameProgrammingClass",// window Class name.
                        "Vector Class demo - By the Programming Ace.",// window name.
                        WS_OVERLAPPEDWINDOW | WS_VISIBLE |// The window style.
                        WS_SYSMENU | WS_CLIPCHILDREN |// window style.
                        WS_CLIPSIBLINGS,// window style.
                        100, 100,// window x, y coordinate.
                        640, 480,// window width and height.
                        NULL,// handle to parent window.
                        NULL,// menu.
                        hInstance,// handle to app instance.
                        NULL);   // pointer to window creation data.

   // If there was an error with creating the window, then close the program.
   if(!hwnd) return 0;

   ShowWindow(hwnd, SW_SHOW); // This shows the window.
   UpdateWindow(hwnd);        // This forces a paint message.

   isFinished = false;        // False = running, True = not running.

   // If initialize fail (return false), then we don't want the program to run.
   if(!InitializeGL()) isFinished = true;

   // This is the messsage loop.
   while(!isFinished)
      {
         if(PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
            {
               // If a quit message is received then stop rendering and quit the app.
               if(msg.message == WM_QUIT)
                  {
                     isFinished = true;
                  }

               TranslateMessage(&msg);
               DispatchMessage(&msg);
            }
         else
            {
               RenderScene();
            }
      }

   return (int)msg.wParam;
}


bool InitializeGL()
{
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

   glShadeModel(GL_SMOOTH);
   glEnable(GL_DEPTH_TEST);


   // Here we will initialize the data for each point.
   // Here we will test one of our other overloaded operators -= sign.  We will -=
   // temp and temp2 which equals 1.0f, 0.0f, -3.0f (for the x, y, and z).  Then
   // we will use the = overloaded sign to set Triangle[0] equal to temp.
   CVector4 temp(2.0f, 4.0f, -2.0f);
   CVector4 temp2(1.0f, 4.0f, 1.0f);

   temp -= temp2;

   //Triangle[0] = CVector4(1.0f, 0.0f, -3.0f);
   Triangle[0] = temp;
   Triangle[1] = CVector4(0.0f, 1.0f, -3.0f);
   Triangle[2] = CVector4(-1.0f, 0.0f, -3.0f);

   return true;
}


void RenderScene()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);    // Clears the screen.
   glLoadIdentity();                   // Reset modelview matrix for new frame.

   // This is a temporary array to store the extended points of the original triangle.
   CVector4 Temp[3];

   // What ExtendVertexPos does is it will move a point (the first parameter) in
   // a direction (which depends on the second parameter), by a certain amount (detemined
   // by the third parameter).  This is used in the shadow volume demo and I am
   // using it here to demonstrate one of the new function in the vector class.  The
   // second param is normally a light position.  Once we get the direction from the
   // current point and the light position we move the point in that direction by a
   // certain amount (the third param).  Its not quite like translating but I guess one
   // can think of it like that a little.
   Temp[0].ExtendVertexPos(Triangle[0], CVector4(0.0f, 0.0f, 1.0f), 3.5f);
   Temp[1].ExtendVertexPos(Triangle[1], CVector4(0.0f, 0.0f, 1.0f), 3.5f);
   Temp[2].ExtendVertexPos(Triangle[2], CVector4(0.0f, 0.0f, 1.0f), 3.5f);

   // Draw the triangle...
   glBegin(GL_TRIANGLES);
      glColor3f(0.6f, 0.0f, 0.8f); glVertex3f(Temp[0].x, Temp[0].y, Temp[0].z);
      glColor3f(0.6f, 0.2f, 0.2f); glVertex3f(Temp[1].x, Temp[1].y, Temp[1].z);
      glColor3f(0.6f, 0.0f, 0.8f); glVertex3f(Temp[2].x, Temp[2].y, Temp[2].z);
   glEnd();

   SwapBuffers(g_HDC); // Display the new frame.
}


// Recap:

// Again there isn't nothing in this demo but a demonstration of one of the functions
// in the new class.  All that is new to the class is a few functions that will aid us in the
// shadow volume, cel-shading, bump mapping, and much more demos in the near future.


// Copyright June 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com